function [Summary,Simu] = SMC_CP_in_TS(y,type_model,arg_model,N_part,nb_for_SMC_sampler,graph)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%% Sequential Monte Carlo sampler for simulating posterior
%%%%%%%%%%% distributions from Sparse CP-AR, CP-ARMA and CP-GARCH models
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%% Inputs
%%%% y: time series Tx1
%%%% type_model: 'AR','ARMA' or 'GARCH' (default 'AR')
%%%% arg_model: structure which details the AR or ARMA model (not taken
%%%% into account for for GARCH processes).
%%%% arg_model.AR_lags = 2  - Includes AR terms in the mean y_{t-1} and y_{t-2} 
%%%% arg_model.MA_lags = 1  - Includes MA term in the mean epsilon_{t-1}
%%%% arg_model.X = Z - includes Z as explanatory variables. Z must be of
%%%% dimension TxK where K denotes the additional variables.
%%%% N_part: Number of SMC particles (default 5000).
%%%% nb_for_SMC_sampler: last in-sample observation --> do a forecast
%%%% exercise starting at observation t=nb_for_SMC_sampler+1.
%%%% graph : =1, it provides graphics of the parameters over time (default =1).
%%%%%%%%%%% Outputs
%%% Summary: A structure with the most important posterior information
%%% Simu: A structure with many posterior summaries and with the posterior
%%% draws.
%%%%%%%%%
%%% Summary.param_over_time : (T x nb_param x nb_quantiles) Quantiles from 5% to 95% of the posterior distributions of
%%% the parameters over time (it includes the variance --- last column).
%%% Summary.nb_regime_per_param : The mode of the number of regimes per
%%% parameter (it includes the variance parameter)
%%% Summary.MLL = Simu.MLL : Estimation of the Marginal log-likelihood.
%%%%%%%%%
%%% Example:
%%% [DGP] = DGP_simulation('AR','B'); %%% Simulation of a series from DGP  'B'
%%% arg_model.AR_lags = 2;
%%% [Summary,Simu] = SMC_CP_in_TS(DGP.y,'AR',arg_model,5000);
%%%%%%%%%
taille = max(size(y));
if(nargin<6)
    graph = 1; %%% provides graphics of the parameters over time.
    if(nargin<5)
        nb_for_SMC_sampler = taille;
        if(nargin<4)
            N_part = 5000;
        end
    end
end
if(sum(strcmp(type_model,{'AR','ARMA','GARCH'}))~=1)
    warning('type model is badly specified --> Estimation of an AR(1) model');
    type_model = 'AR';
    arg_model.AR_lags = 1;
end
do_unshrink = 0;
prob_Pen = 0.95;
addpath('./2MU_functions');
addpath('./Statistical functions');
nb_prop_in_SADE = 10;
if(taille~=size(y,1))
    y = y';
end
% nb_for_SMC_sampler = taille;
switch type_model
    case 'AR'
        addpath('./Estimation_AR');
        if(isstruct(arg_model))
            if(isfield(arg_model,'AR_lags'))
                AR_lags = arg_model.AR_lags;
                disp(['Estimation of an AR(' num2str(AR_lags) ')'])
                intercept = 1;
                if(isfield(arg_model,'X'))
                    [DATA] = AR_explanatory_var(y,1:AR_lags,[],intercept,arg_model.X);
                else
                    [DATA] = AR_explanatory_var(y,1:AR_lags,[],intercept);
                end
                dim_model = size(DATA.X,2) + 1;
                error = 0;
            end
        end
    case 'ARMA'
        addpath('./Estimation ARMA');
        if(isstruct(arg_model))
            if(isfield(arg_model,'MA_lags'))
                MA_lags = arg_model.MA_lags;
                if(isfield(arg_model,'AR_lags'))
                    AR_lags = arg_model.AR_lags;
                    disp(['Estimation of an ARMA(' num2str(AR_lags) ',' num2str(MA_lags) ')'])
                    intercept = 1;
                    if(isfield(arg_model,'X'))
                        [DATA] = AR_explanatory_var(y,1:AR_lags,[],intercept,arg_model.X);
                    else
                        [DATA] = AR_explanatory_var(y,1:AR_lags,[],intercept);
                    end
                    dim_model = size(DATA.X,2) + MA_lags + 1;
                    taille = size(DATA.X,1);
                    DATA.X = [DATA.X zeros(taille,MA_lags)];
                    error = 0;
                end
            end
        end
    case 'GARCH'
        addpath('./Estimation_GARCH');
        error = 0;
        DATA.y = y;
        DATA.X = zeros(max(size(y)),2);
        dim_model = 3;
        clear arg_model;
        arg_model.AR_lags = 1;
        disp('Estimation of a GARCH(1,1)')
end
if(error == 1)
    error('arg model is badly specified for the chosen model'); %#ok<NOEFF>
end
if(min(size(y))>1)
    error('The TS must be univariate'); %#ok<NOEFF>
end

taille = max(size(DATA.y));
nb_forecast = 1;
if(nb_for_SMC_sampler>taille)
    warning('nb for SMC should be smaller than the sample size'); %#ok<*WNTAG>
    nb_for_SMC_sampler = taille;
    nb_forecast = 1;
end
if(nb_forecast>1)
    warning('Above one step ahead, the forecast may be unreliable if AR or MA lags is different from zero');
end
up_to_time = nb_for_SMC_sampler;
if(up_to_time==taille)
    phi_targeted = 0.975;
else
    phi_targeted = 0.95;
end
y_t = DATA.y(1:up_to_time);
X_t = DATA.X(1:up_to_time,:);
if(up_to_time+nb_forecast>taille)
    X_for = zeros(nb_forecast,min(size(X_t)));
    y_for = zeros(nb_forecast,1);
else
    X_for = DATA.X(up_to_time+1:up_to_time+nb_forecast,:);
    y_for = DATA.y(up_to_time+1:up_to_time+nb_forecast);
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%% Initialization of the particles %%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
[Stat_Ng] = compute_NG_statistics_models(y,type_model,arg_model);
Kmax = size(Stat_Ng.Ubounds,1);
regime = Kmax+1;
dim_2MU = dim_model*Kmax;
Ubounds_breaks = Stat_Ng.Ubounds;


prob = prob_Pen; %%% Penalty of the BIC without Bayes factor.
penal_BIC = log((1-prob)/(taille*prob));
penal_dep = -20;
diff_BIC = penal_BIC-penal_dep;
if(do_unshrink==1)
    nb_choice_penal = 5;
    temper_2MU = (1:nb_choice_penal)/nb_choice_penal;
    phi_BIC = exp(penal_dep);
else
    nb_choice_penal = 1;
    temper_2MU = 0;
    phi_BIC = exp(penal_BIC);
end
Ubounds_BIC = [phi_BIC/2 3*phi_BIC/2];

prob_param_in_spike = zeros(dim_model,Kmax,nb_choice_penal);
nb_regime_in_spike = zeros(dim_model,nb_choice_penal);


Choix_quantile = 0.25;
cass_param_a = [Stat_Ng.J;taille];
regime_1estimation = size(cass_param_a,1);
deb = 1;
point_model = zeros(dim_model,regime_1estimation);

mean_param_1est = zeros(dim_model,regime_1estimation);
cov_var_1est = zeros(dim_model,dim_model,regime_1estimation);
N_part_1est = 1000;
if(dim_model>10)
    N_part_1est = 5000;
end
for q=1:regime_1estimation
    arg_model_no_break = arg_model;
    if(isfield(arg_model,'X'))
        arg_model_no_break.X = arg_model.X((deb:cass_param_a(q)),:);
    end
    
    [Simu_no_break] = SMC_TS_no_break(y(deb:cass_param_a(q)),type_model,arg_model_no_break,N_part_1est);
    
    med_param = quantile(Simu_no_break.log_theta,0.5);
    val_param = quantile(Simu_no_break.log_theta,Choix_quantile);
    point = (med_param-val_param)';
    point_model(:,q) = point;
    if(sum(point==0)~=0)
        [Simu_no_break] = SMC_TS_no_break(y(deb:cass_param_a(q)),type_model,arg_model_no_break,10000);
        med_param = quantile(Simu_no_break.log_theta,0.5);
        val_param = quantile(Simu_no_break.log_theta,Choix_quantile);
        point = (med_param-val_param)';
        point_model(:,q) = point;
    end
    mean_param_1est(:,q) = mean(Simu_no_break.log_theta)';
    cov_var_1est(:,:,q) = cov(Simu_no_break.log_theta);
    
    deb = cass_param_a(q)+1;
end
[point,ind_regime] = min(point_model,[],2);
if(sum(point==0)~=0)
    ind = find(point==0);
    nb_prob = max(size(ind));
    for z=1:nb_prob
        point_model(ind(z),ind_regime(ind(z))) = 1e3;
        [point(ind(z))] = min(point_model(ind(z),:));
    end
end
point = kron(ones(Kmax,1),point);
[Ubounds_2MU] = bounds_2MU_creation(point,dim_2MU,type_model);




disp('***************************************')
disp('***************************************')
disp('**  Initialisation of the particles  **')
disp('***************************************')
disp('***************************************')
init_time = tic;
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%% Initialization of the Evolutionary algorithm %%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
find_nb_chain = max(20,2*dim_2MU):N_part;
a = mod(N_part,find_nb_chain);
ind_chain = find(a==0,1);
if(isempty(ind_chain)==1)
    warning('Wrong number of particles'); 
    ind_parfor = 8;
    N_part = ind_parfor*2*dim_2MU;
    nb_chain = 2*dim_2MU;
else
    nb_chain = find_nb_chain(ind_chain); %%% Number of parallel chain into one MCMC of the Rejuvanation step.
    ind_parfor = N_part/nb_chain; %%% Number of parallel MCMCs of the Rejuvanation step.
end



%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%% Initialization of the particles %%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
log_like = zeros(N_part,1);
log_prior = zeros(N_part,4);
particles = zeros(N_part,dim_model);
particles_2MU = zeros(N_part,dim_2MU);
particles_breaks = zeros(N_part,Kmax);
particles_phi = zeros(N_part,dim_2MU);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%% Prior Variances
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
mean_param = zeros(dim_model,1);
var_param = ones(dim_model,1);
switch type_model
    case{'GARCH','MSM'}
        mean_param = [0.5;0;0.5];
        var_param = [2;1.5;2];
    case{'AR','ARMA'}
        var_param(1:dim_model-1) = 10;
        var_param(end) = 15; %%% Bound of the Uniform distribution on ln sigma^2
end

for q=1:N_part %%% Initialization of the particles - Drawing from the prior distributions
    undefined = 1;
    count = 0;
    while(undefined==1)
        log_prior_current = zeros(4,1);
        
        %%% Initialisation of phi
        phi_rnd = Ubounds_BIC(1) + (Ubounds_BIC(2)-Ubounds_BIC(1))*rand(dim_2MU,1);
        particles_phi(q,:) = phi_rnd';
        log_prior_current(4) = -1*dim_2MU*log((Ubounds_BIC(2)-Ubounds_BIC(1)));
        
        %%% Initialisation of 2MU parameters
        [weight] = phi_into_weight(phi_rnd,Ubounds_2MU,Kmax);
        [theta_2MU] = simulation_from_2MU(weight,Ubounds_2MU,Kmax);
        particles_2MU(q,:) = theta_2MU';
        log_prior_current(2) = eval_2MU(particles_2MU(q,:)',weight,Ubounds_2MU);
        
        
        %%% Initialisation of 1st regime parameter
        [log_theta,log_prior_current(1)] = simulation_and_eval_from_prior(0,dim_model,mean_param,var_param,type_model,1,1);
        
        
        %%% Initialisation of break dates
        break_dates = round(Ubounds_breaks(:,1) + (Ubounds_breaks(:,2)-Ubounds_breaks(:,1)).*rand(Kmax,1)); %#ok<*PFBNS>
        log_prior_current(3) = -1*sum(log(Ubounds_breaks(:,2)-Ubounds_breaks(:,1)));
        particles_breaks(q,:) = break_dates';
        
        log_prior(q,:) = log_prior_current';
        particles(q,:) = log_theta';
        log_theta_all = [log_theta;theta_2MU];
        
        [log_like(q)] = launch_likelihood(log_theta_all,break_dates,regime,y_t,X_t,type_model,arg_model);
        count = count +1;
        if(log_like(q)>-1e9 || count>100)
           undefined = 0; 
        end
    end
end




nb_all_iter = 2*taille;
if(nb_all_iter>2000)
    nb_all_iter = 2000;
end
Simu = struct('log_like',zeros(N_part,1),'MLL_all',zeros(nb_all_iter,1),'temper',zeros(nb_all_iter,1),'ESS',zeros(nb_all_iter,2),'accept_rate',zeros(nb_all_iter,nb_prop_in_SADE),'prob_type',zeros(nb_all_iter,nb_prop_in_SADE));
accept_rate_break = zeros(nb_all_iter,2*Kmax);
w_part = ones(N_part,1)/N_part; %% Initialisation of the wieghts of the particles.
MLL = 0;

i=2;
ESS_tab = zeros(nb_all_iter-1,2);
ESS_tab(1,:) = N_part;
SMC_end = 0; %% Notify when the SMC is finished.
sampler = 1; %% if = 1, iterating with SMC sampler otherwise with IBIS.
temper = 0; %% Initialisation of the temper function (phi(1) = 0)
val_iter = 0; %% counter for saving the posterior distribution.


nb_MCMC = 1;
dim_tot = dim_model + dim_2MU;
switch type_model
    case 'GARCH'
        prob_indice = (0.5).^(1:dim_tot);
    otherwise
        prob_indice = (0.5).^(1:dim_tot);
end
prob_indice = prob_indice/sum(prob_indice);

ESS_borne = ceil(0.9*N_part); %%% Threshold which specifies when a resampling step takes place.
option = optimset('MaxIter',1e7,'MaxFunEvals',1e7);



post_log_like = zeros(N_part,nb_all_iter);
post_log_prior = zeros(N_part,4,nb_all_iter);
post_time = zeros(nb_all_iter,1);
post_tau = zeros(nb_all_iter,1);
post_y_for = zeros(N_part,nb_forecast,nb_all_iter);

post_w = zeros(N_part,nb_all_iter);
post_log_theta_1reg = zeros(N_part,dim_model,nb_all_iter);
post_theta_2MU = zeros(N_part,dim_2MU,nb_all_iter);
post_break_dates = zeros(N_part,Kmax,nb_all_iter);
post_theta_phi = zeros(N_part,dim_2MU,nb_all_iter);
post_pred_dens = zeros(nb_forecast,nb_all_iter);
post_pred_dens_simu = zeros(N_part,nb_forecast,nb_all_iter);
post_MLL = zeros(nb_all_iter,1);






dens_stay_aide = zeros(nb_chain,ind_parfor);
prior_S_aide =  zeros(nb_chain,4,ind_parfor);
stay_particles_aide =  zeros(nb_chain,dim_model,ind_parfor);
stay_particles_2MU_aide =  zeros(nb_chain,dim_2MU,ind_parfor);
stay_particles_breaks_aide =  zeros(nb_chain,Kmax,ind_parfor);
stay_particles_phi_aide =  zeros(nb_chain,dim_2MU,ind_parfor);
stay_y_for_aide = zeros(nb_chain,nb_forecast,ind_parfor);
stay_dens_pred_aide = zeros(nb_chain,nb_forecast,ind_parfor);
particles_y_for = zeros(N_part,nb_forecast);
particles_dens_pred_for = zeros(N_part,nb_forecast);

val_prop_ESS_det = 1e-5:1e-4:0.1;

adapt_rate_GARCH_DR = 1.5*ones(3,1);
alpha_adapt_breaks = 0.5*ones(Kmax,1);
ind_t = 1;
prob_type = ones(nb_prop_in_SADE,1)/nb_prop_in_SADE;
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%% Initialisation of the Metropolis-Hasting proposal %%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
time_all_TNT = tic;
Cov_particles = zeros(dim_2MU,dim_2MU,ind_parfor);
ind_temper_2MU = 0;



while(SMC_end==0)
    if(sampler==1) %%% Tempering iteration - E-AIS step
        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        %%%%%%%%%%%%% Correction step of the SMC sampler %%%%%%%%%%%%%%%%%%%%%%%%%%
        %%%%%%%%%%%%% Find the optimal value of phi(i) such that a specific
        %%%%%%%%%%%%% value of ESS is obtained.
        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        if(ind_temper_2MU>0)
            if(ind_temper_2MU==max(size(temper_2MU)))
                sampler = 0;
            end
            
            temper_2MU_current = temper_2MU(ind_temper_2MU);
            new_penal_BIC = penal_dep +diff_BIC*temper_2MU_current;
            phi_BIC = exp(new_penal_BIC);
            disp(['***** Former Uniform Penalty : ' num2str(log(Ubounds_BIC)) '  ***********'])
            Ubounds_BIC = min([0.5*phi_BIC 3*phi_BIC/2],1);
            
            disp(['***** NEW Uniform Penalty : ' num2str(log(Ubounds_BIC)) '  ***********'])
            log_prior_move = zeros(N_part,1);
            particles_phi = Ubounds_BIC(1) + (Ubounds_BIC(2)-Ubounds_BIC(1))*rand(N_part,dim_2MU);
            parfor q=1:N_part
                [weight] = phi_into_weight(particles_phi(q,:)',Ubounds_2MU,Kmax);
                log_prior_move(q) = eval_2MU(particles_2MU(q,:)',weight,Ubounds_2MU);
            end
            w_aux = log_prior_move-log_prior(:,2);
            
            [maxi] = max(w_aux);
            w_aux = w_part.*exp(w_aux-maxi);
            Q = sum(w_aux);
            MLL = MLL + log(Q) + maxi;
            w_part = w_aux/Q;
            log_prior(:,2) = log_prior_move;
            ind_temper_2MU = ind_temper_2MU +1;
            
        else
            ESS_target = floor(phi_targeted*ESS_tab(i-1,1));
            ESS_control = calc_ESS(log_like,w_part,log(val_prop_ESS_det./(1-val_prop_ESS_det)),ESS_target);
            min_part_deviation = 0.1*(N_part-ESS_borne);
            aide = (ESS_control<min_part_deviation);
            if(sum(aide)>0)
                ind = find(aide==1);
                choice_diff = val_prop_ESS_det(ind(end));
            else
                val_opt = zeros(50,2);
                parfor q=1:50
                    if(q<20)
                        val_dep_opt = rand()*0.005;
                    else
                        val_dep_opt = rand()*0.35;
                    end
                    [val1,val2] = fminsearch(@(theta_phi) calc_ESS(log_like,w_part,theta_phi,ESS_target),log(val_dep_opt/(1-val_dep_opt)),option);
                    val_opt(q,:) = [val1 val2];
                end
                [~,ind] = min(val_opt(:,2));
                log_choice_diff = val_opt(ind,1);
                choice_diff = exp(log_choice_diff)/(1+exp(log_choice_diff));
            end

            
            
            val_temp = temper;
            temper = min(temper + choice_diff,1);
            choice_diff = temper-val_temp;
            

            if(temper==1)
                ind_temper_2MU = 1;
                if(do_unshrink==0)
                    sampler = 0;
                end
            end
            w_aux = choice_diff*(log_like);
            [maxi] = max(w_aux);
            w_aux = w_part.*exp(w_aux-maxi);
            Q = sum(w_aux);
            MLL = MLL + log(Q) + maxi;
            w_part = w_aux/Q;
            
        end
    else
        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        %%%%%%%%%%%%% Correction step of the IBIS or the TNT algorithm %%%%%%%%%%%%%%%%%%%%%%%%%%
        %%%%%%%%%%%%% Observations are added one by one.%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        up_to_time = up_to_time + 1;
        ind_t = ind_t +1;
        y_t = DATA.y(1:up_to_time);
        X_t = DATA.X(1:up_to_time,:);
        q_move = log_like;
        parfor z=1:N_part
            log_theta_all = [particles(z,:) particles_2MU(z,:)]';
            [log_like(z)] = launch_likelihood(log_theta_all,particles_breaks(z,:)',regime,y_t,X_t,type_model,arg_model);
        end
        
        w_aux = log_like-q_move;
        [maxi] = max(w_aux);
        w_aux = w_part.*exp(w_aux-maxi);
        Q = sum(w_aux);
        
        
        MLL = MLL + log(Q) + maxi;
        w_part = w_aux/Q;
        if(up_to_time+nb_forecast>taille)
            X_for = zeros(nb_forecast,min(size(X_t)));
            y_for = zeros(nb_forecast,1);
        else
            X_for = DATA.X(up_to_time+1:up_to_time+nb_forecast,:);
            y_for = DATA.y(up_to_time+1:up_to_time+nb_forecast);
        end

    end
    
    
    Simu.temper(i-1) = temper;
    Simu.MLL_all(i-1) = MLL;
    
   
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    %%% Resampling using the stratified approach
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    ESS = 1/sum(w_part.*w_part);
    disp('***************************************')
    disp('***************************************')
    disp(['Temper ' num2str(temper) ' and ESS ' num2str(ESS)])
    disp('***************************************')
    disp(['Nb MCMC : ' num2str(nb_MCMC)])
    disp('***************************************')
    if(ESS<ESS_borne)
        [indice] = multinomialrnd_c(w_part,rand(N_part,1));
        indice = indice(randperm(N_part));
        w_part = ones(N_part,1)/N_part;
        ESS = N_part;
    else
        indice = (1:N_part)';
        indice = indice(randperm(N_part));
    end
    
    
    
    aide_param = particles;
    aide_2MU = particles_2MU;
    aide_breaks = particles_breaks;
    aide_phi = particles_phi;
    
    log_like_aide = log_like;
    log_prior_aide = log_prior;
    log_like = log_like_aide(indice);
    log_prior = log_prior_aide(indice,:);
    particles = aide_param(indice,:);
    particles_2MU = aide_2MU(indice,:);
    particles_breaks = aide_breaks(indice,:);
    particles_phi =  aide_phi(indice,:);
    
    ESS_tab(i,1) = ESS;
    if(ESS_tab(i,2)==0)
        ESS_tab(i,2) = ESS;
    end
    
    
    
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    %%% Initialization of the Mutation step
    %%% by MH, DREAM and D-DREAM algorithms
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    accept_GARCH = zeros(ind_parfor,nb_prop_in_SADE);
    count_GARCH = zeros(ind_parfor,nb_prop_in_SADE);
    for q=1:ind_parfor
        ind = (q-1)*nb_chain;
        dens_stay_aide(:,q) = log_like(ind+1:ind+nb_chain);
        prior_S_aide(:,:,q) = log_prior(ind+1:ind+nb_chain,:);
        stay_particles_aide(:,:,q) = particles(ind+1:ind+nb_chain,:);
        stay_particles_2MU_aide(:,:,q) = particles_2MU(ind+1:ind+nb_chain,:);
        stay_particles_breaks_aide(:,:,q) = particles_breaks(ind+1:ind+nb_chain,:);
        stay_particles_phi_aide(:,:,q) = particles_phi(ind+1:ind+nb_chain,:);
        Cov_particles(:,:,q) = cov(particles_2MU(ind+1:ind+nb_chain,:)) + 0.001*eye(dim_2MU);
    end
    
    
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    %%% Mutation step : each living particle is mutated by an MCMC kernel
    %%% exhibiting as invariant distribution the current distribution of
    %%% the SMC
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    Cov_particles_all_inv = inv(mean(Cov_particles,3));
    Mahalanobis_distance = zeros(ind_parfor,nb_prop_in_SADE);
    
    count_breaks = zeros(ind_parfor,2*Kmax);
    accept_breaks = zeros(ind_parfor,2*Kmax);

    %%%%%%%%%%%%%%%%%%
    %%%% Adaptation of the parameter proposal distribution
    %%%%%%%%%%%%%%%%%% 
    if(temper>0.1)
        param_in_small_comp = zeros(dim_model*Kmax,N_part);
        parfor q=1:N_part %%% Initialization of the particles - Drawing from the prior distributions
            [weight] = phi_into_weight(particles_phi(q,:)',Ubounds_2MU,Kmax);
            [~,small_comp] = eval_2MU(particles_2MU(q,:)',weight,Ubounds_2MU);
            param_in_small_comp(:,q) = small_comp;
        end
        prob_proposal = sum(param_in_small_comp,2)/N_part;
        nb_reg_current = 1 + sum(reshape(prob_proposal,dim_model,Kmax)<0.5,2);
        disp('*********** Idendification regimes ************************');
        disp(nb_reg_current');
    end
    


    time_iter = tic;
    parfor q=1:ind_parfor
        dens_stay = dens_stay_aide(:,q);
        prior_S = prior_S_aide(:,:,q);
        stay_particles = stay_particles_aide(:,:,q)';
        stay_particles_2MU = stay_particles_2MU_aide(:,:,q)';
        stay_particles_breaks = stay_particles_breaks_aide(:,:,q)';
        stay_particles_phi = stay_particles_phi_aide(:,:,q)';
        stay_y_for = zeros(nb_forecast,nb_chain);
        stay_dens_pred = zeros(nb_forecast,nb_chain);
        
        accept_GARCH_parfor = ones(nb_prop_in_SADE,1);
        count_GARCH_parfor = ones(nb_prop_in_SADE,1);
        Mahalanobis_distance_parfor = zeros(nb_prop_in_SADE,1);

        count_breaks_parfor = zeros(2*Kmax,1);
        accept_breaks_parfor = zeros(2*Kmax,1);        
        for M=1:nb_MCMC
            for DR=1:nb_chain
                [weight] = phi_into_weight(stay_particles_phi(:,DR),Ubounds_2MU,Kmax);
                sampling_parameters = 1;
                indice_to_sample = (1:dim_tot)';
                switch type_model
                    case {'ARMA'}
                            tau = [stay_particles_breaks(:,DR);taille];
                            delta_theta = reshape([stay_particles(:,DR);stay_particles_2MU(:,DR)],dim_model,regime);
                            [dens_stay(DR),~,~,~,epsilon] = launch_likelihood([stay_particles(:,DR);stay_particles_2MU(:,DR)],stay_particles_breaks(:,DR),regime,y_t,X_t,type_model,arg_model);
                            
                            [delta_theta,~] = sampling_sigma_parameters(delta_theta,Ubounds_2MU,weight,temper,tau,epsilon);
                            stay_particles_2MU(:,DR) = reshape(delta_theta(1:end,2:end),dim_model*(regime-1),1);
                            stay_particles(:,DR) = delta_theta(:,1);
                            
                            [dens_stay(DR)] = launch_likelihood([stay_particles(:,DR);stay_particles_2MU(:,DR)],stay_particles_breaks(:,DR),regime,y_t,X_t,type_model,arg_model);
                            [~,prior_S(DR,1)] = simulation_and_eval_from_prior(stay_particles(:,DR),dim_model,mean_param,var_param,type_model,0,1);
                            prior_S(DR,2) = eval_2MU(stay_particles_2MU(:,DR),weight,Ubounds_2MU);
                            
                            indice_sampling = ones(dim_model,1);
                            indice_sampling(dim_model) = 0;
                            indice_sampling = logical(kron(ones(regime,1),indice_sampling));
                            indice_to_sample = find(indice_sampling==1);

                    case {'AR'}
                        if(rand()>0.25)
                            sampling_parameters = 0;
                            tau = [stay_particles_breaks(:,DR);taille];
                            delta_theta = reshape([stay_particles(:,DR);stay_particles_2MU(:,DR)],dim_model,regime);
                            [dens_stay(DR),~,~,~,epsilon] = launch_likelihood([stay_particles(:,DR);stay_particles_2MU(:,DR)],stay_particles_breaks(:,DR),regime,y_t,X_t,type_model,arg_model);
                            
                            [delta_theta,~] = sampling_sigma_parameters(delta_theta,Ubounds_2MU,weight,temper,tau,epsilon);
                            [delta_theta,~] = sampling_mean_parameters(y_t,X_t,delta_theta,var_param,Ubounds_2MU,weight,temper,tau);
                            stay_particles_2MU(:,DR) = reshape(delta_theta(1:end,2:end),dim_model*(regime-1),1);
                            stay_particles(:,DR) = delta_theta(:,1);
                            
                            [dens_stay(DR)] = launch_likelihood([stay_particles(:,DR);stay_particles_2MU(:,DR)],stay_particles_breaks(:,DR),regime,y_t,X_t,type_model,arg_model);
                            [~,prior_S(DR,1)] = simulation_and_eval_from_prior(stay_particles(:,DR),dim_model,mean_param,var_param,type_model,0,1);
                            prior_S(DR,2) = eval_2MU(stay_particles_2MU(:,DR),weight,Ubounds_2MU);
                            
                        end
                end
                
                if(sampling_parameters==1)
                    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
                    %%%% Metropolis Move : SADE algorithm (Evolutionary)
                    %%%% Update of all the model parameters in one block
                    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
                    nb_dimension_to_draw = max(size(indice_to_sample));
                    indice_model = randperm(nb_dimension_to_draw);
                    nb_draw = multinomialrnd(prob_indice,rand());
                    
                    nb_iteration = ceil(nb_dimension_to_draw/nb_draw);
                    deb_param = 1;
                    for z=1:nb_iteration
                        end_param = deb_param+nb_draw-1;
                        if(end_param > nb_dimension_to_draw)
                            end_param = nb_dimension_to_draw;
                        end
                        current_indice = zeros(dim_tot,1);
                        current_indice(indice_to_sample(indice_model(deb_param:end_param))) = 1;
                        current_indice = logical(current_indice);
                        
                        stay_all = [stay_particles' stay_particles_2MU'];
                        [move_param_current,log_dens_prop, selection] = SADE_final(stay_all(:,current_indice),dens_stay+sum(prior_S(:,1:2),2),DR,0,nb_chain,M,prob_type,adapt_rate_GARCH_DR);
                        
                        
                        move_particles_all = [stay_particles(:,DR);stay_particles_2MU(:,DR)];
                        move_particles_all(current_indice) = move_param_current;
                        
                        move_particles = move_particles_all(1:dim_model);
                        move_2MU = move_particles_all(dim_model+1:end);
                        count_GARCH_parfor(selection) = count_GARCH_parfor(selection)+1;
                        
                        
                        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
                        %%%% Computation of the log-likelihood
                        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
                        log_prior_move_2MU = eval_2MU(move_2MU,weight,Ubounds_2MU);
                        log_prior_stay_2MU = eval_2MU(stay_particles_2MU(:,DR),weight,Ubounds_2MU);
                        [~,log_prior_move_1reg] = simulation_and_eval_from_prior(move_particles,dim_model,mean_param,var_param,type_model,0,1);
                        [log_like_move] = launch_likelihood(move_particles_all,stay_particles_breaks(:,DR),regime,y_t,X_t,type_model,arg_model);
                        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
                        %%%% Computation of the Metropolis ratio
                        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
                        if(exp(temper*(log_like_move-dens_stay(DR))+log_prior_move_1reg+log_prior_move_2MU-prior_S(DR,1)-log_prior_stay_2MU+log_dens_prop)>rand())
                            dens_stay(DR) = log_like_move;
                            prior_S(DR,1) = log_prior_move_1reg;
                            prior_S(DR,2) = log_prior_move_2MU;
                            vect = stay_particles_2MU(:,DR)-move_2MU;
                            dist = vect'*Cov_particles_all_inv*vect; %#ok<MINV>
                            stay_particles(:,DR) = move_particles;
                            stay_particles_2MU(:,DR) = move_2MU;
                            
                            Mahalanobis_distance_parfor(selection) = Mahalanobis_distance_parfor(selection) + dist; %% Computation of the Mahalanobis distance for the mutation equation
                            accept_GARCH_parfor(selection) = accept_GARCH_parfor(selection)+1;
                        end
                        deb_param = deb_param + nb_draw;
                    end
                end

                
                particles_current = [stay_particles(:,DR);stay_particles_2MU(:,DR)];
                [break_move] = D_DREAM(stay_particles_breaks,DR,0.2,nb_chain,M);
                count_breaks_parfor(1) = count_breaks_parfor(1) +1;
                if(sum(Ubounds_breaks(:,1)>break_move)==0 && sum(Ubounds_breaks(:,2)<break_move)==0)
                    [log_like_move] = launch_likelihood(particles_current,break_move,regime,y_t,X_t,type_model,arg_model);
                    if(exp(temper*(log_like_move-dens_stay(DR)))>rand())
                        dens_stay(DR) = log_like_move;
                        stay_particles_breaks(:,DR) = break_move;
                        accept_breaks_parfor(1) = accept_breaks_parfor(1) +1;
                    end
                end
                
                %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
                %%%% Gibbs Move : Sampling Phi's
                %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
                [stay_particles_phi(:,DR)] = sampling_phi(stay_particles_2MU(:,DR),Ubounds_2MU,Ubounds_BIC);
                
                %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
                %%%% Forecasting
                %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
                if(M==nb_MCMC && temper == 1)
                    [~,~,stay_y_for(:,DR),stay_dens_pred(:,DR)] = launch_likelihood(particles_current,stay_particles_breaks(:,DR),regime,y_t,X_t,type_model,arg_model,X_for,y_for);
                end
            end
            
        end
        
        
        accept_breaks(q,:) = accept_breaks(q,:) + accept_breaks_parfor'; 
        count_breaks(q,:) = count_breaks(q,:) + count_breaks_parfor';
        accept_GARCH(q,:) = accept_GARCH(q,:) + accept_GARCH_parfor';
        count_GARCH(q,:) = count_GARCH(q,:) + count_GARCH_parfor';
        stay_particles_aide(:,:,q) = stay_particles';
        stay_particles_2MU_aide(:,:,q) = stay_particles_2MU';
        stay_particles_breaks_aide(:,:,q) = stay_particles_breaks';
        stay_particles_phi_aide(:,:,q) = stay_particles_phi';
        stay_y_for_aide(:,:,q) = stay_y_for';
        stay_dens_pred_aide(:,:,q) = stay_dens_pred';
        
        dens_stay_aide(:,q) = dens_stay;
        prior_S_aide(:,:,q) = prior_S;
        
        
        Mahalanobis_distance(q,:) = Mahalanobis_distance(q,:) + Mahalanobis_distance_parfor';
        
        
    end
    Mahalanobis_distance = sum(Mahalanobis_distance,1) + 0.001;
    count_GARCH = sum(count_GARCH,1);
    accept_GARCH = sum(accept_GARCH,1);
    
    rate_breaks = sum(accept_breaks,1)./(sum(count_breaks,1));
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    %%%% Computation of the acceptance rates %%%%
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    prob_type = (Mahalanobis_distance./max(count_GARCH,1))/(sum(Mahalanobis_distance./max(count_GARCH,1))); %% Update of the mutation probabilities    
    rate_GARCH = accept_GARCH./count_GARCH;
    jump = (nb_prop_in_SADE-2)/2;
    adapt_rate_GARCH_DR(1) = max(adapt_rate_GARCH_DR(1) + (mean(rate_GARCH(1:jump))'-0.333)/(ind_t^(0.6)),1.01); %%% Adaption of Atchade and Rosenthal for targeting a fixed acceptance rate - For the Stretch mutations
    adapt_rate_GARCH_DR(2) = max(adapt_rate_GARCH_DR(2) + (mean(rate_GARCH(jump+1:2*jump))'-0.333)/(ind_t^(0.6)),1.01); %%% Adaption of Atchade and Rosenthal for targeting a fixed acceptance rate - For the Walk mutations
    adapt_rate_GARCH_DR(3) = max(adapt_rate_GARCH_DR(3) + (mean(rate_GARCH(2*jump+1:end))'-0.333)/(ind_t^(0.6)),0.01); %%% Adaption of Atchade and Rosenthal for targeting a fixed acceptance rate - For the DREAM mutations
    
    for q=1:ind_parfor
        ind = (q-1)*nb_chain;
        log_like(ind+1:ind+nb_chain) = dens_stay_aide(:,q);
        log_prior(ind+1:ind+nb_chain,:) = prior_S_aide(:,:,q);
        particles(ind+1:ind+nb_chain,:) = stay_particles_aide(:,:,q);
        particles_2MU(ind+1:ind+nb_chain,:) = stay_particles_2MU_aide(:,:,q);
        particles_breaks(ind+1:ind+nb_chain,:) = stay_particles_breaks_aide(:,:,q);
        particles_phi(ind+1:ind+nb_chain,:) = stay_particles_phi_aide(:,:,q);
        particles_y_for(ind+1:ind+nb_chain,:) = stay_y_for_aide(:,:,q);
        particles_dens_pred_for(ind+1:ind+nb_chain,:) = stay_dens_pred_aide(:,:,q);
        
    end
    
    
    
    
    
    
    if(mod(i,1)==0)
        disp(['End of iteration ' num2str(i) ' on ' num2str(nb_all_iter)])
        disp(['Time for one iteration ' num2str(toc(time_iter)) ' sec. '])
        disp(['Tempering ' num2str(temper) ' --- Time : ' num2str(up_to_time)])
        disp(['ML : ' num2str(max(log_like)) ' Min ML : ' num2str(min(log_like))])
        disp('*********************************************************')
        disp('********************** Accept Rate  *********************')
        disp('*********************************************************')
        disp(['Accept. model parameters : ' num2str(rate_GARCH)])
        disp('*********************************************************')
        disp(['Accept. Breaks : ' num2str(rate_breaks)])
        disp(['Adapt. prop. Breaks : ' num2str(alpha_adapt_breaks')])
        disp('*********************************************************')
        disp(['Adapt SADE : ' num2str(adapt_rate_GARCH_DR')])
        disp('*********************************************************')
        disp('********************** Break dates  *********************')
        disp('*********************************************************')
        disp(['Post. mean : ' num2str(mean(particles_breaks))])
        disp(['Post. std. : ' num2str(std(particles_breaks))])
    end
    


    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    %%%% Saving the draws and other quantities %%
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    accept_rate_break(i-1,:) = rate_breaks;
    Simu.accept_rate(i-1,:) = rate_GARCH;
    Simu.prob_type(i-1,:) = prob_type;
    if(temper==1)
        if(sampler==0)
            val_iter = val_iter+1;
            post_tau(val_iter) = up_to_time;
            post_log_like(:,val_iter) = log_like;
            post_log_prior(:,:,val_iter) = log_prior;
            post_log_theta_1reg(:,:,val_iter) = particles;
            post_theta_2MU(:,:,val_iter) = particles_2MU;
            post_break_dates(:,:,val_iter) = particles_breaks;
            post_theta_phi(:,:,val_iter) = particles_phi;
            post_MLL(val_iter) = MLL;
            post_time(val_iter) = toc(init_time);
            post_y_for(:,:,val_iter) = particles_y_for;
            post_pred_dens_simu(:,:,val_iter) = particles_dens_pred_for;
            
            T_left = taille-up_to_time;
            if(T_left>nb_forecast)
                for t=1:nb_forecast
                    post_pred_dens(t,val_iter) =  ksdensity(post_y_for(:,t,val_iter),DATA.y(up_to_time+t));
                end
            elseif(T_left>0)
                for t=1:T_left
                    post_pred_dens(t,val_iter) =  ksdensity(post_y_for(:,t,val_iter),DATA.y(up_to_time+t));
                end
            end
            post_w(:,val_iter) = w_part;
        end
        

        if(ind_temper_2MU==1)
            Simu.log_like_no_pen = log_like;
            Simu.log_prior_no_pen = log_prior;
            Simu.log_theta_1reg_no_pen = particles;
            Simu.theta_2MU_no_pen = particles_2MU;
            Simu.break_dates_no_pen = particles_breaks;
            Simu.theta_phi_no_pen = particles_phi;
            Simu.w_no_pen = w_part;
        end
        param_in_small_comp = zeros(dim_model,Kmax,N_part);
        parfor q=1:N_part %%% Initialization of the particles - Drawing from the prior distributions
            [weight] = phi_into_weight(particles_phi(q,:)',Ubounds_2MU,Kmax);
            [~,small_comp] = eval_2MU(particles_2MU(q,:)',weight,Ubounds_2MU);
            param_in_small_comp(:,:,q) = reshape(small_comp,dim_model,Kmax);
        end
        prob_param_in_spike(:,:,ind_temper_2MU) = sum(param_in_small_comp,3)/N_part;
        nb_regime_in_spike(:,ind_temper_2MU) = 1 + sum(prob_param_in_spike(:,:,ind_temper_2MU)<0.5,2);
        
    end
    i = i+1;
    if(i>nb_all_iter)
        disp('depassement du nombre iteration');
        i = nb_all_iter;
    end
    if(up_to_time==taille && sampler==0)
        SMC_end = 1;
    end
end

if(do_unshrink==1)
    Simu.log_like_no_pen = log_like;
    Simu.log_prior_no_pen = log_prior;
    Simu.log_theta_1reg_no_pen = particles;
    Simu.theta_2MU_no_pen = particles_2MU;
    Simu.break_dates_no_pen = particles_breaks;
    Simu.theta_phi_no_pen = particles_phi;
    Simu.w_no_pen = w_part;
        
end

Simu.time_all = toc(time_all_TNT);
Simu.rate_breaks = rate_breaks;

Simu.count_GARCH = count_GARCH;
Simu.accept_GARCH = accept_GARCH;
Simu.ESS = ESS_tab;
total_iter = i-2;
Simu.accept_rate_break = accept_rate_break(1:total_iter-1,:);
Simu.MLL_all = Simu.MLL_all(1:total_iter);
Simu.temper = Simu.temper(1:total_iter);
Simu.tau = post_tau(1:val_iter);
Simu.log_like = post_log_like(:,1:val_iter);
Simu.log_prior = post_log_prior(:,:,1:val_iter);
Simu.log_theta_1reg = post_log_theta_1reg(:,:,1:val_iter);
Simu.theta_2MU = post_theta_2MU(:,:,1:val_iter);
Simu.break_dates = post_break_dates(:,:,1:val_iter);
Simu.theta_phi = post_theta_phi(:,:,1:val_iter);
Simu.MLL = post_MLL(1:val_iter);
Simu.time = post_time(1:val_iter);
Simu.y_for = post_y_for(:,:,1:val_iter);
Simu.w = post_w(:,1:val_iter);
Simu.post_pred_dens = post_pred_dens(:,1:val_iter);
if(val_iter>1)
    Simu.post_pred_dens_simu = post_pred_dens_simu(:,:,1:val_iter);
end




Simu.ESS = Simu.ESS(1:total_iter-1,:);
Simu.accept_rate = Simu.accept_rate(1:total_iter-1,:);
Simu.prob_type = Simu.prob_type(1:total_iter-1,:);
Simu.adapt_rate_GARCH_DR = adapt_rate_GARCH_DR;

theta = zeros(N_part,regime,dim_model);
param_t = zeros(N_part,up_to_time,dim_model);
parfor q=1:N_part %%% Initialization of the particles - Drawing from the prior distributions
    [~,param_current] = launch_likelihood([particles(q,:) particles_2MU(q,:)]',particles_breaks(q,:)',regime,y_t,X_t,type_model,arg_model);
    theta(q,:,:) = param_current';
    break_date = [particles_breaks(q,:) up_to_time];
    [sn] = trans_sn(break_date,up_to_time);
    for i=1:dim_model
       param_t(q,:,i) = param_current(i,sn); 
    end
end


quant = 0.05:0.05:0.95;
dim_quant = max(size(quant));
param_over_time = zeros(up_to_time,dim_model,dim_quant);
for i=1:dim_model
    for j=1:dim_quant
        a = quantile(param_t(:,:,i),quant(j));
        param_over_time(:,i,j) = a';
    end
end
Simu.prob_param_in_spike = prob_param_in_spike;
Simu.nb_regime_in_spike = nb_regime_in_spike;

Simu.param_in_small_comp = param_in_small_comp;
Simu.param_t = param_over_time;
Simu.theta = theta;
Simu.Simu_no_break = Simu_no_break;
Simu.Stat_Ng = Stat_Ng;
Simu.type_model = type_model;
Simu.arg_model = arg_model;
Simu.y = y_t;
Simu.X = X_t;
Simu.Ubounds_2MU = Ubounds_2MU;
[Simu] = find_best_configuration(Simu);

Summary.param_over_time = Simu.param_t;
Summary.nb_regime_per_param = Simu.nb_regime_in_spike;
Summary.MLL = Simu.MLL;

if(graph==1)
    nb_param = size(Simu.param_t,2);
    nb_even = nb_param;
    if(mod(nb_param,2)~=0)
        nb_even = nb_even +1;
    end
    if(strcmp(type_model,'GARCH')==1)
       Name_GARCH = {'\omega','\alpha','\beta'}; 
    end
    for i=1:nb_param
        subplot(nb_even/2,2,i),plot(squeeze(Simu.param_t(:,i,10)),'linewidth',2),hold on,plot(squeeze(Simu.param_t(:,i,1)),'-.','linewidth',1.5),plot(squeeze(Simu.param_t(:,i,end)),'-.','linewidth',1.5);
        if(strcmp(type_model,'GARCH')==1)
            title(Name_GARCH{i});
        else
            if(i~=nb_param)
                title(['Param ' num2str(i)]);
            else
                title('\sigma_t^2');
            end
        end
    end
end